# Lattesh 
## A Blender Add-on for mesh-based lattice processing

The *Lattesh* Add-on contains a set of tools for the creation and manipulation
of lattices in [Blender](https://www.blender.org). There are software tools 
with good support for lattices, but those are proprietary and expensive. 
This Add-on provides a basic alternative.

The Add-on is programed in Python. It is designed to process rectangular lattices
of cells composed of strait cylindrical struts.
### Lattices

The design was focused on speed and ease of lattice processing. This was achieved
by representing lattices as skeletons, were struts are just edges connecting two 
vertices. A watertight mesh giving the lattice a volume is only created as a last
step in the lattice processing.

Internally, lattices are represented in the Add-on as (degeneraded) meshes with only 
vertices and edges. The standard mesh object from Blender is used. The generated 
lattices can be manipulated using common Blender tools, but lacking faces and 
having an unsusual topology make this editing limited.

### Features

The main features of the add-on are lattice creation, volume filling, and final
mesh generation. In creating a lattice, the user can choose the grid dimensions, 
that is, the number of cells in each direction, the cell size and the cell type. 
All these parameters are available in the add-on’s graphic interface. A lattice 
is then generated in the form of a rectangular regular grid of cubic cells of 
the chosen cell type. 

New cell types can be easily added via a cell definition file, where the cell 
geometry is described in a text-based file format.

More complex lattices can be generated using volume filling. In this feature, 
a watertight mesh representing the desired form of the lattice is taken as input.
The add-on then fills this volume with a lattice of the chosen cell geometry.

Finally, a 3D print-ready lattice can be obtained by giving a thickness to a 
lattice skeleton. In this feature, each edge in the input lattice skeleton 
will generate a cylindrical strut with a user-defined radius. Strut unions
in vertices are geometrically correct and correspond to geometrical boolean 
unions of the cylindrical struts. The generated mesh is watertight and care
was taken to avoid redundant faces and vertices, leading to a smaller mesh size.
The resulting mesh is significantly smaller than what can be obtained by an 
algorithm like marching cubes.

The geometrical boolean operations are computed using the 
Open CASCADE Technology library, an open source geometrical library. 
These operations are computationally expensive, so the Add-on uses the 
regularity of the lattices to reuse the result of these operations on similar 
vertices. This leads to very efficient computation on regular lattices. 
The boolean operations are also notoriously unreliable. The Add-on then detects
when these fail and takes measures to correct them when needed.

### User interface

The Add-on provides a graphical user interface in the form of a panel in the 
side bar of the 3D Viewport. The panel is depicted below.

![Panel image](doc/side_panel.png)

#### Rectangular lattice generation

The top part contains the parameters for regular lattice generation. The 
parameters are:
 - **Cell size**: the length defining the individual cells. 
 - **Cell type**: the cell type defines the topology of the individual cells.
 - **Number of cells**: lattice grid dimensions, that is, how many cells will 
   be on the *x*, *y*, and *z* directions.

Clicking on the **Create Mesh Lattice** will create a rectangular lattice with 
these parameters. 

#### Filling a volume with lattice

If a mesh object is selected, clicking on **Fill Mesh Lattice** will create a 
regular lattice of *cell size* and *cell type* that fills the selected object. 
The boolean intersection algorithm is experimental and can fail. In that case, 
a lattice is still created and must be manually edited to remove the parts outside
the target mesh.

#### Cylindrical struts

If a lattice is selected (or any mesh object), an **Opencascade fill** menu is
available. Upon clicking on **Lattice Volume (OCC)**, the lattice will be given
a volume, where for each edge in the original object a cylindrical strut will 
be created. The strut radius is available as an input parameter. 

This operation will take only some seconds for very regular lattices, but a large
irregular lattice can take several hours to process. In this case, the add-on prints
messages on the terminal relating the progress.

#### Variable strut radius

It is possible to generate struts with varing thickness in a single lattice. To
that end, the mesh must have a float attribute called *radius*, with domain *'POINTS'*.
When such a parameter is present in the selected mesh, an extra option,
*Variable radius*, becomes usable. When checked, the radius of the generated mesh
at each lattice node will be given by this parameter.

The *radius* parameter can be generated using [geometry nodes](https://docs.blender.org/manual/en/latest/modeling/geometry_nodes/index.html)
in Blender. An example is shown below.

![Nodes example](doc/nodes.png)

![Variable radius mesh](doc/result.png)


### Installation

The Add-on is installed as usual in Blender via the *Preferences* menu. However, 
the python distribution in Blender needs extra packages. These are:
 - NumPy
 - SciPy
 - [PythonOCC](https://github.com/tpaviot/pythonocc)
 
While the former two can be easily installed via `pip`, the latter needs either 
building from source other installing with *conda*. See the installation intructions
in *PythonOCC*. If the *conda* route is used, Blender will need to access the *conda* 
python environment. This can be achieved by creating a *conda* environment with 
the required packages and copying it to the place where Blender keeps its python 
distribution, after moving or deleting the original folder. The *conda* python
 version (major and minor) must match the one shipped with Blender.